# -*- coding: utf-8 -*-

# This file is part of HeCalc, which calculates (U-Th)/He dates and uncertainties
# Copyright (C) 2021 Peter E. Martin <pemartin92@gmail.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

'''
Functions to return a (U-Th)/He date or multiple dates.
All functions will read in radionuclides, He, and Ft values
as integers, floats, or numpy arrays (if multiple dates are desired).

All functions return date(s) in years.
'''

import numpy as np

l_U238 = 1.55125e-10
l_U235 = 9.8485e-10
l_Th232 = 4.9475e-11
l_Sm147 = 6.539e-12
l_tot = [l_U238, l_U235, l_Th232, l_Sm147]

def iterated_date(He, t_guess,
                  U238=0, U235=None, Th232=0, Sm147=0,
                  Ft238=1, Ft235=1, Ft232=1, Ft147=1):
    '''
    Return (U-Th)/He date accurately calculated using the Newton-Raphson
    method. The iteration is run until successive iterations differ
    by less than one year. If an array of dates is being calculated,
    the iteration will be run until the averaged difference between iterations
    is less than one year, while eliminating any array elements that
    do not converge.

    Takes int, float, or numpy arrays as input.
    
    Returned date(s) are in years.
    
    There is a restriction on this function of 100 iterations.
    When calculations take longer than this, it is generally because
    the He age equation is not well-behaved (e.g., a radionuclide value
    is negative). In this case, a NaN value is returned.
    
    Note that at least one radionuclide value must be specified. If all
    nuclides are set to 0, nan is returned
    
    Parameters
    ----------
    He : float or array-like, optional
        The amount of helium measured. The exact units are unimportant
        as long as they are consistent with the other radionuclides
    
    t_guess : float or array-like, optional
        An estimation of the date to be calculated. A good approximation
        can be developed from the Meesters and Dunai method, available in
        this module via the meesters_dunai() function
    
    U238 : float or array-like, optional
        The amount of 238U measured
        
    U235 : float or array-like, optional
        The amount of 2385U measured. If no 235U is provided, it is assumed
        to be present at the standard ratio of 137.818 238U/235U
        
    Th232 : float or array-like, optional
        The amount of Th232 measured
        
    Sm147 : float or array-like, optional
        The amount of Sm147 measured
    
    Ft238 : float or array-like, optional
        The alpha ejection correction for 238U. Should be between 0-1.
        
    Ft235 : float or array-like, optional
        The alpha ejection correction for 235U. Should be between 0-1.
        
    Ft232 : float or array-like, optional
        The alpha ejection correction for 232Th. Should be between 0-1.
        
    Ft147 : float or array-like, optional
        The alpha ejection correction for 147Sm. Should be between 0-1.
        
    Returns
    -------
    float or array-like of date(s) in years
    '''
    # if U235 is left with the default value, assume it must be
    # based on a constant ratio to U238
    if U235 is None:
        U235 = U238/137.818
    # If no radionuclides were passed, return all nans
    if np.all(U238 == 0) and np.all(U235 == 0) and np.all(Th232 == 0) and np.all(Sm147 == 0):
        if type(t_guess) != np.ndarray:
            return np.nan
        else:
            nan_return = np.empty(len(He))
            nan_return.fill(np.nan)
            return nan_return
    # First convert ints or floats to arrays of len(1) 
    # to accomodate array datatype
    single_date = False
    if type(t_guess) != np.ndarray:
        single_date = True
        t_guess = np.array([t_guess])
    # create list of current and previous time (with initial
    # value of 0) to track amount of change between iterations
    # print(t_guess)
    times = [np.zeros(len(t_guess)), np.zeros(len(t_guess)), t_guess]
    n = 0
    ignores = []
    # restricting to n<100 is really just a safeguard. In practice, most iterated
    # dates should require <10 cycles to calculate
    # suppress NaN warnings because NaNs are tracked independently and removed.
    # Allowing NaNs results in much better performance
    with np.errstate(divide='ignore', invalid='ignore', over='ignore'):
        while np.nanmean(np.delete(abs(times[2]-times[1]),ignores)) > 1 and n<100:
            n+=1
            times[0] = times[1]
            times[1] = times[2]
            f_t = (8*Ft238*U238 * (np.exp(times[2]*l_U238) - 1) +
                   7*Ft235*U235 * (np.exp(times[2]*l_U235) - 1) +
                   6*Ft232*Th232 * (np.exp(times[2]*l_Th232) - 1) +
                   Ft147*Sm147 * (np.e**(times[2]*l_Sm147) - 1))-He
            f_t_prime = (8*Ft238*l_U238*U238*np.exp(times[2]*l_U238) +
                         7*Ft235*l_U235*U235*np.exp(times[2]*l_U235) +
                         6*Ft232*l_Th232*Th232*np.exp(times[2]*l_Th232) +
                         Ft147*l_Sm147*Sm147*np.exp(times[2]*l_Sm147))
            new_date = times[2] - f_t/f_t_prime
            times[2] = new_date
            # give the method three cycles to converge, and then stop evaluating
            # samples that aren't converging, defined as reversal of convergence
            # direction after three cycles, constrained to changes of >1 year to
            # avoid floating point errors. The with statement here prevents a divide
            # by zero warning, which should arise if the code is working properly
            # (i.e., when there is no change along the array, many elements will be subtracted
            # from themselves and divided)
            if n>3:
                # print(times)
                nonconv = (times[1]-times[2])/(times[0]-times[1])
                new_times = times[2]-times[1]
                new_ignores = np.where((nonconv<0) & (abs(new_times)>1))[0]
                for i in new_ignores:
                    if i not in ignores:
                        ignores.append(i)
    
    # remove non-converged, NaN, and INF values
    final_date = np.delete(times[2], ignores)
    final_date = final_date[~np.isnan(final_date)]
    final_date = final_date[~np.isinf(final_date)]
    
    # If no dates converged, return NaN
    if len(final_date) == 0:
        return np.nan
    # Convert back to float if array was not passed originally
    if single_date:
        final_date = float(final_date[0])
    # print(final_date)
    return final_date

def meesters_dunai(He,
                   U238=0, U235=None, Th232=0, Sm147=0,
                   Ft238=1, Ft235=1, Ft232=1, Ft147=1):
    '''
    Calculate a (U-Th)/He date using the method of Meesters and Dunai (2005)[1].
    Takes floats, ints, or numpy arrays for
    U238, U235, Th232, Sm147, Ft238, Ft235, Ft232, Ft147, and He.
    
    Users are referred to Meesters and Dunai (2005) for details and
    limitations on this method. In short, this method is very efficient
    and reasonably accurate for dates <500 Ma. Older dates (or dates
    where high accuracy is needed) should be calculated using the
    iterated_date() function instead.
                                                            
    If non-physical values are passed to this function (e.g., negative
    production rates of He), a linearized estimate of date is returned
    instead using the equation t = He/TotalProduction.
    
    Note that at least one radionuclide value must be specified. If all
    nuclides are set to 0, nan is returned
        
    Parameters
    ----------
    He : float or array-like, optional
        The amount of helium measured. The exact units are unimportant
        as long as they are consistent with the other radionuclides
    
    U238 : float or array-like, optional
        The amount of 238U measured
        
    U235 : float or array-like, optional
        The amount of 2385U measured. If no 235U is provided, it is assumed
        to be present at the standard ratio of 137.818 238U/235U
        
    Th232 : float or array-like, optional
        The amount of Th232 measured
        
    Sm147 : float or array-like, optional
        The amount of Sm147 measured
    
    Ft238 : float or array-like, optional
        The alpha ejection correction for 238U. Should be between 0-1.
        
    Ft235 : float or array-like, optional
        The alpha ejection correction for 235U. Should be between 0-1.
        
    Ft232 : float or array-like, optional
        The alpha ejection correction for 232Th. Should be between 0-1.
        
    Ft147 : float or array-like, optional
        The alpha ejection correction for 147Sm. Should be between 0-1.
        
    Returns
    -------
    float or array-like of non-iterative date(s) in years
    
    References
    ----------
    [1] Meesters, A.G.C.A., Dunai, T.J., 2005. A noniterative solution of the 
    (U-Th)/He age equation. Geochemistry, Geophysics, Geosystems 6.
    https://doi.org/10.1029/2004GC000834
    '''
    if U235 is None:
        U235 = U238/137.818
    if np.all(U238 == 0) and np.all(U235 == 0) and np.all(Th232 == 0) and np.all(Sm147 == 0):
        if type(He) != np.ndarray:
            return np.nan
        else:
            nan_return = np.empty(len(He))
            nan_return.fill(np.nan)
            return nan_return
    # Calculate individual production values
    p238 = 8*l_U238*U238*Ft238
    p235 = 7*l_U235*U235*Ft235
    p232 = 6*l_Th232*Th232*Ft232
    p147 = l_Sm147*Sm147*Ft147
    # Generate total and weighted mean production
    p_tot = [p238, p235, p232, p147]
    P = p238 + p235 + p232 + p147
    l_wm = sum([l_tot[i]*p_tot[i]/P for i in range(len(p_tot))])
    # Get the Meesters and Dunai estimate
    # With highly uncertain nuclide values, negative
    # log values will sometimes arise. Suppress the
    # warning output and allow the NaN values to carry
    # through and be removed in the final steps
    with np.errstate(invalid='ignore'):
        MD_date = 1/l_wm*np.log((l_wm/P)*He+1)
    return MD_date
        
# Function to get age given He, radioisotopes, and Fts
def get_date(He,
             U238=0, U235=None, Th232=0, Sm147=0,
             Ft238=1, Ft235=1, Ft232=1, Ft147=1):
    '''
    Calculate raw (uncorrected) and alpha-ejection corrected
    dates. Using the Newton-Raphson method to calculate dates
    with date esimtation provided by the Meesters and Dunai method.
    These are returned in a diction with keys "raw date" and
    "corrected date"
    
    Returned dates are in years.
    
    Takes int, float, or numpy arrays as input.
    
    Parameters
    ----------
    He : float or array-like, optional
        The amount of helium measured. The exact units are unimportant
        as long as they are consistent with the other radionuclides
    
    U238 : float or array-like, optional
        The amount of 238U measured
        
    U235 : float or array-like, optional
        The amount of 2385U measured. If no 235U is provided, it is assumed
        to be present at the standard ratio of 137.818 238U/235U
        
    Th232 : float or array-like, optional
        The amount of Th232 measured
        
    Sm147 : float or array-like, optional
        The amount of Sm147 measured
    
    Ft238 : float or array-like, optional
        The alpha ejection correction for 238U. Should be between 0-1.
        
    Ft235 : float or array-like, optional
        The alpha ejection correction for 235U. Should be between 0-1.
        
    Ft232 : float or array-like, optional
        The alpha ejection correction for 232Th. Should be between 0-1.
        
    Ft147 : float or array-like, optional
        The alpha ejection correction for 147Sm. Should be between 0-1.
        
    Returns
    -------
    float or array-like of date(s) in years
    '''
    if U235 is None:
        U235 = U238/137.818
    if np.all(U238 == 0) and np.all(U235 == 0) and np.all(Th232 == 0) and np.all(Sm147 == 0):
        if type(He) != np.ndarray:
            nan_return = np.nan
        else:
            nan_return = np.empty(len(He))
            nan_return.fill(np.nan)
        return {'raw date': nan_return, 'corrected date': nan_return}
    # Use M&D 2005 age as first guess for iterative age
    # start by calculating raw age using 1 for all Ft
    t_app_raw = meesters_dunai(He, U238, U235, Th232, Sm147)
    if type(U238) == np.ndarray:
        if len(U238) == 0:
            return [np.nan, np.nan]
    t_it = iterated_date(He, t_app_raw,
                         U238, U235, Th232, Sm147)
    
    # Repeat with Fts to get corrected age
    t_app_corr = meesters_dunai(He,
                                U238, U235, Th232, Sm147,
                                Ft238, Ft235, Ft232, Ft147)
    if type(U238) == np.ndarray:
        if len(U238) == 0:
            return [np.nan, np.nan]
    t_it_corr = iterated_date(He, t_app_corr,
                              U238, U235, Th232, Sm147,
                              Ft238, Ft235, Ft232, Ft147)
    return {'raw date': t_it, 'corrected date': t_it_corr}


from .date_calculation import meesters_dunai, iterated_date, get_date
from .linear_propagation import date_uncertainty, date_uncertainty_with235
from .montecarlo import monte_carlo
from .main import hecalc_main

# Numerical version:
__version_info__ = (1, 0, 1)
__version__ = '.'.join(map(str, __version_info__))

__author__ = 'Peter E. Martin <peter.martin-2@colorado.edu>'



# -*- coding: utf-8 -*-

# This file is part of HeCalc, which calculates (U-Th)/He dates and uncertainties
# Copyright (C) 2021 Peter E. Martin <pemartin92@gmail.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

'''This file was generated using QtDesigner to construct the GUI'''

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(836, 793)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(14)
        MainWindow.setFont(font)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem = QtWidgets.QSpacerItem(28, 30, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.goButton = QtWidgets.QPushButton(self.centralwidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.goButton.setFont(font)
        self.goButton.setObjectName("goButton")
        self.horizontalLayout_3.addWidget(self.goButton)
        spacerItem1 = QtWidgets.QSpacerItem(48, 30, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.cancelButton = QtWidgets.QPushButton(self.centralwidget)
        self.cancelButton.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.cancelButton.setFont(font)
        self.cancelButton.setObjectName("cancelButton")
        self.horizontalLayout_3.addWidget(self.cancelButton)
        spacerItem2 = QtWidgets.QSpacerItem(28, 30, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.gridLayout_2.addLayout(self.horizontalLayout_3, 3, 0, 2, 1)
        self.progressBar = QtWidgets.QProgressBar(self.centralwidget)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setTextVisible(False)
        self.progressBar.setObjectName("progressBar")
        self.gridLayout_2.addWidget(self.progressBar, 2, 1, 2, 1)
        self.statusMessenger = QtWidgets.QLabel(self.centralwidget)
        self.statusMessenger.setAlignment(QtCore.Qt.AlignCenter)
        self.statusMessenger.setObjectName("statusMessenger")
        self.gridLayout_2.addWidget(self.statusMessenger, 0, 1, 1, 1)
        self.userUpdate = QtWidgets.QTextBrowser(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(12)
        self.userUpdate.setFont(font)
        self.userUpdate.setObjectName("userUpdate")
        self.gridLayout_2.addWidget(self.userUpdate, 1, 1, 1, 1)
        self.progressLabel = QtWidgets.QLabel(self.centralwidget)
        self.progressLabel.setText("")
        self.progressLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.progressLabel.setObjectName("progressLabel")
        self.gridLayout_2.addWidget(self.progressLabel, 4, 1, 1, 1)
        self.scrollArea_3 = QtWidgets.QScrollArea(self.centralwidget)
        self.scrollArea_3.setMinimumSize(QtCore.QSize(0, 100))
        self.scrollArea_3.setWidgetResizable(True)
        self.scrollArea_3.setObjectName("scrollArea_3")
        self.scrollAreaWidgetContents_3 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_3.setGeometry(QtCore.QRect(0, 0, 367, 674))
        self.scrollAreaWidgetContents_3.setObjectName("scrollAreaWidgetContents_3")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents_3)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_6 = QtWidgets.QLabel(self.scrollAreaWidgetContents_3)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.label_6.setFont(font)
        self.label_6.setAlignment(QtCore.Qt.AlignCenter)
        self.label_6.setObjectName("label_6")
        self.gridLayout_3.addWidget(self.label_6, 0, 0, 1, 1)
        self.tabWidget = QtWidgets.QTabWidget(self.scrollAreaWidgetContents_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.tabWidget.setFont(font)
        self.tabWidget.setUsesScrollButtons(False)
        self.tabWidget.setObjectName("tabWidget")
        self.fileTab = QtWidgets.QWidget()
        self.fileTab.setObjectName("fileTab")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.fileTab)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.scrollArea = QtWidgets.QScrollArea(self.fileTab)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 323, 327))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.gridLayout_7 = QtWidgets.QGridLayout()
        self.gridLayout_7.setObjectName("gridLayout_7")
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_7.addItem(spacerItem3, 2, 1, 1, 1)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem4)
        self.outputFileButton = QtWidgets.QPushButton(self.scrollAreaWidgetContents)
        self.outputFileButton.setMaximumSize(QtCore.QSize(16777215, 16777214))
        font = QtGui.QFont()
        font.setPointSize(14)
        self.outputFileButton.setFont(font)
        self.outputFileButton.setObjectName("outputFileButton")
        self.horizontalLayout_10.addWidget(self.outputFileButton)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem5)
        self.gridLayout_7.addLayout(self.horizontalLayout_10, 3, 1, 1, 1)
        self.pat = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        self.pat.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.pat.setFont(font)
        self.pat.setObjectName("pat")
        self.gridLayout_7.addWidget(self.pat, 6, 1, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(13, 30, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem6, 1, 2, 1, 1)
        spacerItem7 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_7.addItem(spacerItem7, 4, 1, 1, 1)
        self.outputHistCheck = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.outputHistCheck.setFont(font)
        self.outputHistCheck.setObjectName("outputHistCheck")
        self.gridLayout_7.addWidget(self.outputHistCheck, 5, 1, 1, 1)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem8)
        self.inputFileButton = QtWidgets.QPushButton(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.inputFileButton.setFont(font)
        self.inputFileButton.setObjectName("inputFileButton")
        self.horizontalLayout_9.addWidget(self.inputFileButton)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem9)
        self.gridLayout_7.addLayout(self.horizontalLayout_9, 1, 1, 1, 1)
        spacerItem10 = QtWidgets.QSpacerItem(13, 30, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem10, 1, 0, 1, 1)
        spacerItem11 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_7.addItem(spacerItem11, 7, 1, 1, 1)
        spacerItem12 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_7.addItem(spacerItem12, 0, 1, 1, 1)
        self.verticalLayout_7.addLayout(self.gridLayout_7)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout_4.addWidget(self.scrollArea)
        self.tabWidget.addTab(self.fileTab, "")
        self.manualTab = QtWidgets.QWidget()
        self.manualTab.setObjectName("manualTab")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.manualTab)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.scrollArea_2 = QtWidgets.QScrollArea(self.manualTab)
        self.scrollArea_2.setWidgetResizable(True)
        self.scrollArea_2.setObjectName("scrollArea_2")
        self.scrollAreaWidgetContents_2 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_2.setGeometry(QtCore.QRect(0, 0, 323, 327))
        self.scrollAreaWidgetContents_2.setObjectName("scrollAreaWidgetContents_2")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents_2)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.gridLayout_8 = QtWidgets.QGridLayout()
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.label_50 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_50.setFont(font)
        self.label_50.setAlignment(QtCore.Qt.AlignCenter)
        self.label_50.setObjectName("label_50")
        self.gridLayout_8.addWidget(self.label_50, 5, 0, 1, 1)
        self.ft232Inbox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.ft232Inbox.setFont(font)
        self.ft232Inbox.setObjectName("ft232Inbox")
        self.gridLayout_8.addWidget(self.ft232Inbox, 9, 2, 1, 1)
        self.ft235sInBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.ft235sInBox.setFont(font)
        self.ft235sInBox.setObjectName("ft235sInBox")
        self.gridLayout_8.addWidget(self.ft235sInBox, 8, 4, 1, 1)
        self.line_16 = QtWidgets.QFrame(self.scrollAreaWidgetContents_2)
        self.line_16.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_16.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_16.setObjectName("line_16")
        self.gridLayout_8.addWidget(self.line_16, 1, 0, 1, 5)
        self.u235inBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        self.u235inBox.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.u235inBox.setFont(font)
        self.u235inBox.setObjectName("u235inBox")
        self.gridLayout_8.addWidget(self.u235inBox, 4, 2, 1, 1)
        self.label_44 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_44.setFont(font)
        self.label_44.setAlignment(QtCore.Qt.AlignCenter)
        self.label_44.setObjectName("label_44")
        self.gridLayout_8.addWidget(self.label_44, 10, 0, 1, 1)
        self.label_48 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_48.setFont(font)
        self.label_48.setAlignment(QtCore.Qt.AlignCenter)
        self.label_48.setObjectName("label_48")
        self.gridLayout_8.addWidget(self.label_48, 9, 0, 1, 1)
        self.label_45 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.label_45.setMinimumSize(QtCore.QSize(0, 38))
        self.label_45.setAlignment(QtCore.Qt.AlignCenter)
        self.label_45.setObjectName("label_45")
        self.gridLayout_8.addWidget(self.label_45, 0, 4, 1, 1)
        self.label_55 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_55.setFont(font)
        self.label_55.setAlignment(QtCore.Qt.AlignCenter)
        self.label_55.setObjectName("label_55")
        self.gridLayout_8.addWidget(self.label_55, 3, 0, 1, 1)
        self.label_47 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.label_47.setAlignment(QtCore.Qt.AlignCenter)
        self.label_47.setObjectName("label_47")
        self.gridLayout_8.addWidget(self.label_47, 0, 2, 1, 1)
        self.ft232sInBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.ft232sInBox.setFont(font)
        self.ft232sInBox.setObjectName("ft232sInBox")
        self.gridLayout_8.addWidget(self.ft232sInBox, 9, 4, 1, 1)
        self.heInBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.heInBox.setFont(font)
        self.heInBox.setObjectName("heInBox")
        self.gridLayout_8.addWidget(self.heInBox, 2, 2, 1, 1)
        self.label_53 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_53.setFont(font)
        self.label_53.setAlignment(QtCore.Qt.AlignCenter)
        self.label_53.setObjectName("label_53")
        self.gridLayout_8.addWidget(self.label_53, 4, 0, 1, 1)
        self.ft238InBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.ft238InBox.setFont(font)
        self.ft238InBox.setObjectName("ft238InBox")
        self.gridLayout_8.addWidget(self.ft238InBox, 7, 2, 1, 1)
        self.label_52 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_52.setFont(font)
        self.label_52.setAlignment(QtCore.Qt.AlignCenter)
        self.label_52.setObjectName("label_52")
        self.gridLayout_8.addWidget(self.label_52, 6, 0, 1, 1)
        self.ft238sInBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.ft238sInBox.setFont(font)
        self.ft238sInBox.setObjectName("ft238sInBox")
        self.gridLayout_8.addWidget(self.ft238sInBox, 7, 4, 1, 1)
        self.label_46 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_46.setFont(font)
        self.label_46.setAlignment(QtCore.Qt.AlignCenter)
        self.label_46.setObjectName("label_46")
        self.gridLayout_8.addWidget(self.label_46, 2, 0, 1, 1)
        self.thInBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.thInBox.setFont(font)
        self.thInBox.setObjectName("thInBox")
        self.gridLayout_8.addWidget(self.thInBox, 5, 2, 1, 1)
        self.ft147Inbox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.ft147Inbox.setFont(font)
        self.ft147Inbox.setObjectName("ft147Inbox")
        self.gridLayout_8.addWidget(self.ft147Inbox, 10, 2, 1, 1)
        self.uInBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        self.uInBox.setEnabled(True)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.uInBox.setFont(font)
        self.uInBox.setObjectName("uInBox")
        self.gridLayout_8.addWidget(self.uInBox, 3, 2, 1, 1)
        self.line_18 = QtWidgets.QFrame(self.scrollAreaWidgetContents_2)
        self.line_18.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_18.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_18.setObjectName("line_18")
        self.gridLayout_8.addWidget(self.line_18, 2, 1, 9, 1)
        self.label_54 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.label_54.setAlignment(QtCore.Qt.AlignCenter)
        self.label_54.setObjectName("label_54")
        self.gridLayout_8.addWidget(self.label_54, 0, 0, 1, 1)
        self.u235sinBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        self.u235sinBox.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.u235sinBox.setFont(font)
        self.u235sinBox.setObjectName("u235sinBox")
        self.gridLayout_8.addWidget(self.u235sinBox, 4, 4, 1, 1)
        self.ft235InBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.ft235InBox.setFont(font)
        self.ft235InBox.setObjectName("ft235InBox")
        self.gridLayout_8.addWidget(self.ft235InBox, 8, 2, 1, 1)
        self.label_49 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_49.setFont(font)
        self.label_49.setAlignment(QtCore.Qt.AlignCenter)
        self.label_49.setObjectName("label_49")
        self.gridLayout_8.addWidget(self.label_49, 7, 0, 1, 1)
        self.hesInBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.hesInBox.setFont(font)
        self.hesInBox.setObjectName("hesInBox")
        self.gridLayout_8.addWidget(self.hesInBox, 2, 4, 1, 1)
        self.smInBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.smInBox.setFont(font)
        self.smInBox.setObjectName("smInBox")
        self.gridLayout_8.addWidget(self.smInBox, 6, 2, 1, 1)
        self.ft147sInBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.ft147sInBox.setFont(font)
        self.ft147sInBox.setObjectName("ft147sInBox")
        self.gridLayout_8.addWidget(self.ft147sInBox, 10, 4, 1, 1)
        self.line_19 = QtWidgets.QFrame(self.scrollAreaWidgetContents_2)
        self.line_19.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_19.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_19.setObjectName("line_19")
        self.gridLayout_8.addWidget(self.line_19, 0, 3, 1, 1)
        self.line_20 = QtWidgets.QFrame(self.scrollAreaWidgetContents_2)
        self.line_20.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_20.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_20.setObjectName("line_20")
        self.gridLayout_8.addWidget(self.line_20, 2, 3, 9, 1)
        self.thsInBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.thsInBox.setFont(font)
        self.thsInBox.setObjectName("thsInBox")
        self.gridLayout_8.addWidget(self.thsInBox, 5, 4, 1, 1)
        self.line_17 = QtWidgets.QFrame(self.scrollAreaWidgetContents_2)
        self.line_17.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_17.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_17.setObjectName("line_17")
        self.gridLayout_8.addWidget(self.line_17, 0, 1, 1, 1)
        self.usInBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.usInBox.setFont(font)
        self.usInBox.setObjectName("usInBox")
        self.gridLayout_8.addWidget(self.usInBox, 3, 4, 1, 1)
        self.smsInBox = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.smsInBox.setFont(font)
        self.smsInBox.setObjectName("smsInBox")
        self.gridLayout_8.addWidget(self.smsInBox, 6, 4, 1, 1)
        self.label_51 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_51.setFont(font)
        self.label_51.setAlignment(QtCore.Qt.AlignCenter)
        self.label_51.setObjectName("label_51")
        self.gridLayout_8.addWidget(self.label_51, 8, 0, 1, 1)
        self.gridLayout_8.setColumnStretch(0, 20)
        self.gridLayout_8.setColumnStretch(1, 1)
        self.gridLayout_8.setColumnStretch(2, 30)
        self.gridLayout_8.setColumnStretch(3, 1)
        self.gridLayout_8.setColumnStretch(4, 40)
        self.verticalLayout_9.addLayout(self.gridLayout_8)
        self.scrollArea_2.setWidget(self.scrollAreaWidgetContents_2)
        self.verticalLayout_8.addWidget(self.scrollArea_2)
        self.tabWidget.addTab(self.manualTab, "")
        self.gridLayout_3.addWidget(self.tabWidget, 1, 0, 1, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem13 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem13)
        self.U235_measuredCheck = QtWidgets.QCheckBox(self.scrollAreaWidgetContents_3)
        self.U235_measuredCheck.setText("")
        self.U235_measuredCheck.setObjectName("U235_measuredCheck")
        self.horizontalLayout_4.addWidget(self.U235_measuredCheck)
        self.label_3 = QtWidgets.QLabel(self.scrollAreaWidgetContents_3)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_4.addWidget(self.label_3)
        spacerItem14 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem14)
        self.gridLayout_3.addLayout(self.horizontalLayout_4, 2, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(self.scrollAreaWidgetContents_3)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.monteCarloCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.monteCarloCheckBox.setChecked(True)
        self.monteCarloCheckBox.setObjectName("monteCarloCheckBox")
        self.gridLayout.addWidget(self.monteCarloCheckBox, 0, 0, 1, 1)
        self.linearCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.linearCheckBox.setChecked(True)
        self.linearCheckBox.setObjectName("linearCheckBox")
        self.gridLayout.addWidget(self.linearCheckBox, 1, 0, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox, 3, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem15 = QtWidgets.QSpacerItem(13, 43, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem15)
        self.label_2 = QtWidgets.QLabel(self.scrollAreaWidgetContents_3)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.label_2.setFont(font)
        self.label_2.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        self.precisionEnter = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.precisionEnter.setFont(font)
        self.precisionEnter.setDecimals(3)
        self.precisionEnter.setSingleStep(0.001)
        self.precisionEnter.setProperty("value", 0.01)
        self.precisionEnter.setObjectName("precisionEnter")
        self.horizontalLayout.addWidget(self.precisionEnter)
        spacerItem16 = QtWidgets.QSpacerItem(13, 43, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem16)
        self.gridLayout_3.addLayout(self.horizontalLayout, 4, 0, 1, 1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem17 = QtWidgets.QSpacerItem(18, 30, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem17)
        self.decLabel = QtWidgets.QLabel(self.scrollAreaWidgetContents_3)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.decLabel.setFont(font)
        self.decLabel.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.decLabel.setWordWrap(True)
        self.decLabel.setObjectName("decLabel")
        self.horizontalLayout_2.addWidget(self.decLabel)
        self.decBox = QtWidgets.QSpinBox(self.scrollAreaWidgetContents_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.decBox.setFont(font)
        self.decBox.setMaximum(10)
        self.decBox.setProperty("value", 2)
        self.decBox.setObjectName("decBox")
        self.horizontalLayout_2.addWidget(self.decBox)
        spacerItem18 = QtWidgets.QSpacerItem(17, 30, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem18)
        self.gridLayout_3.addLayout(self.horizontalLayout_2, 5, 0, 1, 1)
        self.scrollArea_3.setWidget(self.scrollAreaWidgetContents_3)
        self.gridLayout_2.addWidget(self.scrollArea_3, 0, 0, 3, 1)
        self.gridLayout_2.setColumnStretch(0, 5)
        self.gridLayout_2.setColumnStretch(1, 6)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 836, 22))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        self.menubar.setFont(font)
        self.menubar.setObjectName("menubar")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionHelp = QtWidgets.QAction(MainWindow)
        self.actionHelp.setObjectName("actionHelp")
        self.actionExit = QtWidgets.QAction(MainWindow)
        self.actionExit.setObjectName("actionExit")
        self.menuHelp.addAction(self.actionHelp)
        self.menuHelp.addAction(self.actionAbout)
        self.menuFile.addAction(self.actionExit)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "HeCalc"))
        self.goButton.setText(_translate("MainWindow", "Go!"))
        self.cancelButton.setText(_translate("MainWindow", "Cancel"))
        self.statusMessenger.setText(_translate("MainWindow", "Status"))
        self.label_6.setText(_translate("MainWindow", "Input Type"))
        self.outputFileButton.setText(_translate("MainWindow", "Save Output As"))
        self.pat.setText(_translate("MainWindow", "Parameterize Histograms"))
        self.outputHistCheck.setText(_translate("MainWindow", "Output Histograms"))
        self.inputFileButton.setText(_translate("MainWindow", "Choose Input File"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.fileTab), _translate("MainWindow", "From File"))
        self.label_50.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" vertical-align:super;\">232</span>Th</p></body></html>"))
        self.ft232Inbox.setText(_translate("MainWindow", "1.0"))
        self.ft235sInBox.setText(_translate("MainWindow", "0.0"))
        self.u235inBox.setText(_translate("MainWindow", "1.0"))
        self.label_44.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" vertical-align:super;\">147</span>F<span style=\" vertical-align:sub;\">T</span></p></body></html>"))
        self.label_48.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" vertical-align:super;\">232</span>F<span style=\" vertical-align:sub;\">T</span></p></body></html>"))
        self.label_45.setText(_translate("MainWindow", "Absolute\n"
"Uncertainty (1σ)"))
        self.label_55.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" vertical-align:super;\">238</span>U</p></body></html>"))
        self.label_47.setText(_translate("MainWindow", "Value"))
        self.ft232sInBox.setText(_translate("MainWindow", "0.0"))
        self.heInBox.setText(_translate("MainWindow", "1.0"))
        self.label_53.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" vertical-align:super;\">235</span>U</p></body></html>"))
        self.ft238InBox.setText(_translate("MainWindow", "1.0"))
        self.label_52.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" vertical-align:super;\">147</span>Sm</p></body></html>"))
        self.ft238sInBox.setText(_translate("MainWindow", "0.0"))
        self.label_46.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" vertical-align:super;\">4</span>He</p></body></html>"))
        self.thInBox.setText(_translate("MainWindow", "1.0"))
        self.ft147Inbox.setText(_translate("MainWindow", "1.0"))
        self.uInBox.setText(_translate("MainWindow", "1.0"))
        self.label_54.setText(_translate("MainWindow", "Variable"))
        self.u235sinBox.setText(_translate("MainWindow", "0.0"))
        self.ft235InBox.setText(_translate("MainWindow", "1.0"))
        self.label_49.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" vertical-align:super;\">238</span>F<span style=\" vertical-align:sub;\">T</span></p></body></html>"))
        self.hesInBox.setText(_translate("MainWindow", "0.0"))
        self.smInBox.setText(_translate("MainWindow", "1.0"))
        self.ft147sInBox.setText(_translate("MainWindow", "0.0"))
        self.thsInBox.setText(_translate("MainWindow", "0.0"))
        self.usInBox.setText(_translate("MainWindow", "0.0"))
        self.smsInBox.setText(_translate("MainWindow", "0.0"))
        self.label_51.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" vertical-align:super;\">235</span>F<span style=\" vertical-align:sub;\">T</span></p></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.manualTab), _translate("MainWindow", "Manual"))
        self.label_3.setText(_translate("MainWindow", "Check if <sup>235</sup>U was measured directly"))
        self.groupBox.setTitle(_translate("MainWindow", "Modeling Type"))
        self.monteCarloCheckBox.setText(_translate("MainWindow", "Monte Carlo"))
        self.linearCheckBox.setText(_translate("MainWindow", "Linear Error Propagation"))
        self.label_2.setText(_translate("MainWindow", "<html><head/><body><p align=\"center\">Enter desired precision<br/>of Monte Carlo uncertainty in %</p></body></html>"))
        self.decLabel.setText(_translate("MainWindow", "<html><head/><body><p align=\"center\">Enter number of decimals to report</p></body></html>"))
        self.menuHelp.setTitle(_translate("MainWindow", "Help"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.actionAbout.setText(_translate("MainWindow", "About"))
        self.actionHelp.setText(_translate("MainWindow", "Help"))
        self.actionExit.setText(_translate("MainWindow", "Exit"))


# -*- coding: utf-8 -*-
# This file is part of HeCalc, which calculates (U-Th)/He dates and uncertainties
# Copyright (C) 2021 Peter E. Martin <pemartin92@gmail.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

'''
This module contains the explicit math to calculate linear
uncertainty propagation for the (U-Th)/He age system.

Two functions are provided for this calculation:
"date_uncertainty" and "date_uncertainty_with235"

"date_uncertainty" should be used in all cases where 235U
was not measured directly but instead inferred from U or 238U
concentration. If 235U was measured directly, "date_uncertainty_with235"
should be used instead.

A number of helper functions are also present in this module.
These explicitly calculate the first derivatives of the age equation
with respect to date to propagate uncertainty. They are not imported
directly for the HeCalc package, but may be used independently if needed.

A large number of arguments are available for this module.
Required positional arguments:
    He, t
Optional arguments:
U238, U235, Th232, Sm147, Ft238, Ft235, Ft232, Ft147,
He_s, U238_s, U235_s (only if U235 was measured) ,Th232_s, Sm147_s,
Ft238_s, Ft235_s, Ft232_s, Ft147_s
where U238_s is uncertainty in U238 for example.
All radionuclides and uncertainties default to a value of 0,
while all Fts default to a value of 1.

Note that at least one radionuclide value must be specified.
'''

import numpy as np

l_U238 = 1.55125e-10
l_U235 = 9.8485e-10
l_Th232 = 4.9475e-11
l_Sm147 = 6.539e-12

def _age_func_prime(U238, U235, Th232, Sm147,
                    Ft238, Ft235, Ft232, Ft147,
                    t):
    return ((8*Ft238*l_U238*U238*np.e**(t*l_U238) +
             7*Ft235*l_U235*U235*np.e**(t*l_U235) +
             6*Ft232*l_Th232*Th232*np.e**(t*l_Th232) +
             Ft147*l_Sm147*Sm147*np.e**(t*l_Sm147)))

def _He_prime(U238, U235, Th232, Sm147,
              Ft238, Ft235, Ft232, Ft147,
              He, t):
    return (1/_age_func_prime(U238, U235, Th232, Sm147, Ft238, Ft235, Ft232, Ft147, t))

def _U238_prime_noU235(U238, U235, Th232, Sm147,
                       Ft238, Ft235, Ft232, Ft147,
                       He, t):
    return (
        -(8*Ft238*(np.e**(l_U238*t)-1)+(7/137.818)*Ft235*(np.e**(l_U235*t)-1))/
        _age_func_prime(U238, U235, Th232, Sm147, Ft238, Ft235, Ft232, Ft147, t)
        )

def _U238_prime_withU235(U238, U235, Th232, Sm147,
                         Ft238, Ft235, Ft232, Ft147,
                         He, t):
    return (
        -(8*Ft238*(np.e**(l_U238*t)-1))/
        _age_func_prime(U238, U235, Th232, Sm147, Ft238, Ft235, Ft232, Ft147, t)
        )

def _U235_prime(U238, U235, Th232, Sm147,
                Ft238, Ft235, Ft232, Ft147,
                He, t):
    return (
        -(7*Ft235*(np.e**(l_U235*t)-1))/
        _age_func_prime(U238, U235, Th232, Sm147, Ft238, Ft235, Ft232, Ft147, t)
        )

def _Th232_prime(U238, U235, Th232, Sm147,
                 Ft238, Ft235, Ft232, Ft147,
                 He, t):
    return (
        -(6*Ft232*(np.e**(l_Th232*t)-1))/
        _age_func_prime(U238, U235, Th232, Sm147, Ft238, Ft235, Ft232, Ft147, t)
        )

def _Sm147_prime(U238, U235, Th232, Sm147,
                 Ft238, Ft235, Ft232, Ft147,
                 He, t):
    return (
        -(Ft147*(np.e**(l_Sm147*t)-1))/
        _age_func_prime(U238, U235, Th232, Sm147, Ft238, Ft235, Ft232, Ft147, t)
        )

def _Ft238_prime(U238, U235, Th232, Sm147,
                 Ft238, Ft235, Ft232, Ft147,
                 He, t):
    return (
        -(8*U238*(np.e**(l_U238*t)-1))/
        _age_func_prime(U238, U235, Th232, Sm147, Ft238, Ft235, Ft232, Ft147, t)
        )

def _Ft235_prime(U238, U235, Th232, Sm147,
                 Ft238, Ft235, Ft232, Ft147,
                 He, t):
    return (
        -(7*U235*(np.e**(l_U235*t)-1))/
        _age_func_prime(U238, U235, Th232, Sm147, Ft238, Ft235, Ft232, Ft147, t)
        )

def _Ft232_prime(U238, U235, Th232, Sm147,
                 Ft238, Ft235, Ft232, Ft147,
                 He, t):
    return (
        -(6*Th232*(np.e**(l_Th232*t)-1))/
        _age_func_prime(U238, U235, Th232, Sm147, Ft238, Ft235, Ft232, Ft147, t)
        )

def _Ft147_prime(U238, U235, Th232, Sm147,
                 Ft238, Ft235, Ft232, Ft147,
                 He, t):
    return (
        -(Sm147*(np.e**(l_Sm147*t)-1))/
        _age_func_prime(U238, U235, Th232, Sm147, Ft238, Ft235, Ft232, Ft147, t)
        )

def date_uncertainty(He, t, He_s=0,
                     U238=0, U235=None, Th232=0, Sm147=0,
                     Ft238=1, Ft235=1, Ft232=1, Ft147=1,
                     U238_s=0, Th232_s=0, Sm147_s=0,
                     Ft238_s=0, Ft235_s=0, Ft232_s=0, Ft147_s=0,
                     U238_Th232_v=0, U238_Sm147_v=0, Th232_Sm147_v=0,
                     Ft238_Ft235_v=0, Ft238_Ft232_v=0, Ft238_Ft147_v=0,
                     Ft235_Ft232_v=0, Ft235_Ft147_v=0, Ft232_Ft147_v=0):
    '''
    Returns symmetrical uncertainty via linear uncertainty propagation
    for a given date with uncertainty in He, radionuclides, and/or Ft values.
    
    Accepts integers, floats, or numpy arrays.
    
    WARNING:
    The exact date is required; this function does *not* calculate
    date explicitly. If an incorrect date is provided, the uncertainty
    calculation will also be incorrect.
    
    ~THIS FUNCTION SHOULD ONLY BE USED IF 235U IS INFERRED FROM 238U~
    If U235 was measured directly, the function date_uncertainty_with235()
    should be used instead.
    U235 is provided as an optional argument only if one wishes
    to use a non-standard value for the natural U ratio. If U235
    was measured directly, it will have its own uncertainty value
    and date_uncertainty_with235() should be used instead.
    
    Note that at least one radionuclide value must be specified.
    
    Parameters
    ----------
    He : float or array-like, optional
        The amount of helium measured. The exact units are unimportant
        as long as they are consistent with the other radionuclides
    
    t : float or array-like, optional
        The date for the uncertainty being calculated. This should be
        accurately calculated (e.g., using the get_date() function)
    
    He_s : float or array-like, optional
        The uncertainty in He, in the same units
        
    U238 : float or array-like, optional
        The amount of 238U measured
        
    U235 : float or array-like, optional
        The amount of 2385U measured. If no 235U is provided, it is assumed
        to be present at the standard ratio of 137.818 238U/235U
        
    Th232 : float or array-like, optional
        The amount of Th232 measured
        
    Sm147 : float or array-like, optional
        The amount of Sm147 measured
    
    Ft238 : float or array-like, optional
        The alpha ejection correction for 238U. Should be between 0-1.
        
    Ft235 : float or array-like, optional
        The alpha ejection correction for 235U. Should be between 0-1.
        
    Ft232 : float or array-like, optional
        The alpha ejection correction for 232Th. Should be between 0-1.
        
    Ft147 : float or array-like, optional
        The alpha ejection correction for 147Sm. Should be between 0-1.
    
    U238_s : float or array-like, optional
        The uncertainty in 238U

    Th232_s : float or array-like, optional
        The uncertainty in Th232
        
    Sm147_s : float or array-like, optional
        The uncertainty in Sm147
        
    Ft238_s : float or array-like, optional
        The uncertainty in Ft238
        
    Ft235_s : float or array-like, optional
        The uncertainty in Ft235
        
    Ft232_s : float or array-like, optional
        The uncertainty in Ft232
        
    Ft147_s : float or array-like, optional
        The uncertainty in Ft147
    
    U238_Th232_v : float or array-like, optional
        The covariance between U238 and Th232
    
    U238_Sm147_v : float or array-like, optional
        The covariance between U238 and Sm147
    
    Th232_Sm147_v : float or array-like, optional
        The covariance between Th232 and Sm147
    
    Ft238_Ft235_v : float or array-like, optional
        The covariance between Ft238 and Ft235
    
    Ft238_Ft232_v : float or array-like, optional
        The covariance between Ft238 and Ft232
    
    Ft238_Ft147_v : float or array-like, optional
        The covariance between Ft238 and Ft147
    
    Ft235_Ft232_v : float or array-like, optional
        The covariance between Ft235 and Ft232
    
    Ft235_Ft147_v : float or array-like, optional
        The covariance between Ft235 and Ft147
    
    Ft232_Ft147_v : float or array-like, optional
        The covariance between Ft232 and Ft147
        
    Returns
    -------
    float or array-like of uncertainty values
    '''
    if U235 is None:
        U235 = U238/137.818
    dHe = _He_prime(U238, U235, Th232, Sm147,
                    Ft238, Ft235, Ft232, Ft147,
                    He, t)
    dU238_no235 = _U238_prime_noU235(U238, U235, Th232, Sm147,
                                       Ft238, Ft235, Ft232, Ft147,
                                       He, t)
    dTh232 = _Th232_prime(U238, U235, Th232, Sm147,
                          Ft238, Ft235, Ft232, Ft147,
                          He, t)
    dSm147 = _Sm147_prime(U238, U235, Th232, Sm147,
                          Ft238, Ft235, Ft232, Ft147,
                          He, t)
    dFt238 = _Ft238_prime(U238, U235, Th232, Sm147,
                          Ft238, Ft235, Ft232, Ft147,
                          He, t)
    dFt235 = _Ft235_prime(U238, U235, Th232, Sm147,
                  Ft238, Ft235, Ft232, Ft147,
                  He, t)
    dFt232 = _Ft232_prime(U238, U235, Th232, Sm147,
                          Ft238, Ft235, Ft232, Ft147,
                          He, t)
    dFt147 = _Ft147_prime(U238, U235, Th232, Sm147,
                  Ft238, Ft235, Ft232, Ft147,
                  He, t)
    
    return (
        (dHe * He_s)**2+
        (dU238_no235 * U238_s)**2+
        (dTh232 * Th232_s)**2+
        (dSm147 * Sm147_s)**2+
        (dFt238 * Ft238_s)**2+
        (dFt235 * Ft147_s)**2+
        (dFt232 * Ft232_s)**2+
        (dFt147 * Ft147_s)**2+
        2*(dU238_no235 * dTh232 * U238_Th232_v)+
        2*(dU238_no235 * dSm147 * U238_Sm147_v)+
        2*(dTh232 * dSm147 * Th232_Sm147_v)+
        2*(dFt238 * dFt235 * Ft238_Ft235_v)+
        2*(dFt238 * dFt232 * Ft238_Ft232_v)+
        2*(dFt238 * dFt147 * Ft238_Ft147_v)+
        2*(dFt235 * dFt232 * Ft235_Ft232_v)+
        2*(dFt235 * dFt147 * Ft235_Ft147_v)+
        2*(dFt232 * dFt147 * Ft232_Ft147_v)
        )**(1/2)

def date_uncertainty_with235(He, t, He_s=0,
                             U238=0, U235=0, Th232=0, Sm147=0,
                             Ft238=1, Ft235=1, Ft232=1, Ft147=1,
                             U238_s=0, U235_s=0, Th232_s=0, Sm147_s=0,
                             Ft238_s=0, Ft235_s=0, Ft232_s=0, Ft147_s=0,
                             U238_U235_v=0, U238_Th232_v=0, U238_Sm147_v=0,
                             U235_Th232_v=0, U235_Sm147_v=0, Th232_Sm147_v=0,
                             Ft238_Ft235_v=0, Ft238_Ft232_v=0, Ft238_Ft147_v=0,
                             Ft235_Ft232_v=0, Ft235_Ft147_v=0, Ft232_Ft147_v=0):
    '''
    Returns symmetrical uncertainty for a given date with uncertainty
    in He, radionuclides, and/or all Ft values.
    
    Accepts integers, floats, or numpy arrays.
    
    WARNING:
    The exact date is required; this function does *not* calculate
    date explicitly. If an incorrect date is provided, the uncertainty
    calculation will also be incorrect.
    
        
    ~THIS FUNCTION SHOULD ONLY BE USED IF 235U HAS BEEN MEASURED DIRECTLY~
    If 235U is instead inferred from 238U measurement, the function
    "date_uncertainty" should be used instead.
    
    Note that at least one radionuclide value must be specified.
    
    Parameters
    ----------
    He : float or array-like, optional
        The amount of helium measured. The exact units are unimportant
        as long as they are consistent with the other radionuclides
    
    t : float or array-like, optional
        The date for the uncertainty being calculated. This should be
        accurately calculated (e.g., using the get_date() function)
    
    He_s : float or array-like, optional
        The uncertainty in He, in the same units
        
    U238 : float or array-like, optional
        The amount of 238U measured
        
    U235 : float or array-like, optional
        The amount of 2385U measured. If no 235U is provided, it is assumed
        to be present at the standard ratio of 137.818 238U/235U
        
    Th232 : float or array-like, optional
        The amount of Th232 measured
        
    Sm147 : float or array-like, optional
        The amount of Sm147 measured
    
    Ft238 : float or array-like, optional
        The alpha ejection correction for 238U. Should be between 0-1.
        
    Ft235 : float or array-like, optional
        The alpha ejection correction for 235U. Should be between 0-1.
        
    Ft232 : float or array-like, optional
        The alpha ejection correction for 232Th. Should be between 0-1.
        
    Ft147 : float or array-like, optional
        The alpha ejection correction for 147Sm. Should be between 0-1.
    
    U238_s : float or array-like, optional
        The uncertainty in 238U
        
    U235_s : float or array-like, optional
        The uncertainty in 235U, if measured.

    Th232_s : float or array-like, optional
        The uncertainty in Th232
        
    Sm147_s : float or array-like, optional
        The uncertainty in Sm147
        
    Ft238_s : float or array-like, optional
        The uncertainty in Ft238
        
    Ft235_s : float or array-like, optional
        The uncertainty in Ft235
        
    Ft232_s : float or array-like, optional
        The uncertainty in Ft232
        
    Ft147_s : float or array-like, optional
        The uncertainty in Ft147
        
    U238_U235_v : float or array-like, optional
        The covariance between U238 and U235
    
    U238_Th232_v : float or array-like, optional
        The covariance between U238 and Th232
    
    U238_Sm147_v : float or array-like, optional
        The covariance between U238 and Sm147
    
    U235_Th232_v : float or array-like, optional
        The covariance between U235 and Th232
    
    U235_Sm147_v : float or array-like, optional
        The covariance between U235 and Sm147
    
    Th232_Sm147_v : float or array-like, optional
        The covariance between Th232 and Sm147
    
    Ft238_Ft235_v : float or array-like, optional
        The covariance between Ft238 and Ft235
    
    Ft238_Ft232_v : float or array-like, optional
        The covariance between Ft238 and Ft232
    
    Ft238_Ft147_v : float or array-like, optional
        The covariance between Ft238 and Ft147
    
    Ft235_Ft232_v : float or array-like, optional
        The covariance between Ft235 and Ft232
    
    Ft235_Ft147_v : float or array-like, optional
        The covariance between Ft235 and Ft147
    
    Ft232_Ft147_v : float or array-like, optional
        The covariance between Ft232 and Ft147
        
    Returns
    -------
    float or array-like of uncertainty values
    '''
    dHe = _He_prime(U238, U235, Th232, Sm147,
                    Ft238, Ft235, Ft232, Ft147,
                    He, t)
    dU238_with235 = _U238_prime_withU235(U238, U235, Th232, Sm147,
                                         Ft238, Ft235, Ft232, Ft147,
                                         He, t)
    dU235 = _U235_prime(U238, U235, Th232, Sm147,
                        Ft238, Ft235, Ft232, Ft147,
                        He, t)
    dTh232 = _Th232_prime(U238, U235, Th232, Sm147,
                          Ft238, Ft235, Ft232, Ft147,
                          He, t)
    dSm147 = _Sm147_prime(U238, U235, Th232, Sm147,
                          Ft238, Ft235, Ft232, Ft147,
                          He, t)
    dFt238 = _Ft238_prime(U238, U235, Th232, Sm147,
                          Ft238, Ft235, Ft232, Ft147,
                          He, t)
    dFt235 = _Ft235_prime(U238, U235, Th232, Sm147,
                  Ft238, Ft235, Ft232, Ft147,
                  He, t)
    dFt232 = _Ft232_prime(U238, U235, Th232, Sm147,
                          Ft238, Ft235, Ft232, Ft147,
                          He, t)
    dFt147 = _Ft147_prime(U238, U235, Th232, Sm147,
                  Ft238, Ft235, Ft232, Ft147,
                  He, t)
    
    return (
        (dHe * He_s)**2+
        (dU238_with235 * U238_s)**2+
        (dU235 * U235_s)**2+
        (dTh232 * Th232_s)**2+
        (dSm147 * Sm147_s)**2+
        (dFt238 * Ft238_s)**2+
        (dFt235 * Ft147_s)**2+
        (dFt232 * Ft232_s)**2+
        (dFt147 * Ft147_s)**2+
        2*(dU238_with235 * dU235 * U238_U235_v)+
        2*(dU238_with235 * dTh232 * U238_Th232_v)+
        2*(dU238_with235 * dSm147 * U238_Sm147_v)+
        2*(dU235 * dTh232 * U235_Th232_v)+
        2*(dU235 * dSm147 * U235_Sm147_v)+
        2*(dTh232 * dSm147 * Th232_Sm147_v)+
        2*(dFt238 * dFt235 * Ft238_Ft235_v)+
        2*(dFt238 * dFt232 * Ft238_Ft232_v)+
        2*(dFt238 * dFt147 * Ft238_Ft147_v)+
        2*(dFt235 * dFt232 * Ft235_Ft232_v)+
        2*(dFt235 * dFt147 * Ft235_Ft147_v)+
        2*(dFt232 * dFt147 * Ft232_Ft147_v)
        )**(1/2)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hecalc', 'hecalc.GUI']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.18.1',
 'openpyxl>=3.0.3',
 'pandas>=1.0.3',
 'scipy>=1.4.1',
 'setuptools>=41.2.0']

extras_require = \
{'gui': ['PyQt5>=5.15.6,<6.0.0',
         'tabulate>=0.8.9,<0.9.0',
         'xlrd>=1.2.0,<2.0.0']}

setup_kwargs = {
    'name': 'hecalc',
    'version': '1.0.1',
    'description': 'A tool for performing (U-Th)/He data reduction and uncertainty propagation',
    'long_description': None,
    'author': 'Peter E. Martin',
    'author_email': 'peter.martin-2@colorado.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
# This file is part of HeCalc, which calculates (U-Th)/He dates and uncertainties
# Copyright (C) 2021 Peter E. Martin <pemartin92@gmail.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""
This module takes a dataset and runs Monte Carlo uncertainty modeling.

It contains functions for the Monte Carlo modeling and to generate
histograms and parameterize those histograms if a user so desires.
"""

from scipy.stats import skewnorm as sk
import numpy as np
from .date_calculation import get_date

def make_histogram(mc, parameterize):
    '''
    Create histograms and potentially parameterize them as skew-normal
    probability distribution functions.
    
    Parameters
    ----------
    mc : Array-like
        The list of numbers to place in a histogram
        
    parameterize : bool
        whether to parameterize the histogram
        
    Returns
    -------
    Generates a dictionary with the histogram and potentially parameters.
    the histogram is returned as a list of the bin centers and a list of
    the number of samples within the bin. The parameters areassociated
    with the key "fit distribution", and are a list of the three parameters
    (skewness, location, scale) to calculate a skew-normal distribution
    '''
    # Generate histogram with bins centered
    bins = int(len(mc)/1000)
    if bins>1000:
        bins = 1000
    elif bins<10:
        bins = 10
    histo = list(np.histogram(mc, bins=bins))[::-1]
    histo[0] = (histo[0][1:] + histo[0][:-1]) / 2
    
    # Get stats for skewnormal function if the user called for parameterization
    if parameterize:
        # Subsample Monte Carlo results if lots of simulations were run.
        # skewnorm.fit is very slow with large samples, and the accuracy of
        # the fit is perfectly acceptable with 1e5 samples
        if len(mc) > int(1e5):
            sample = np.random.choice(mc, int(1e5), replace = False)
        else:
            sample = mc
        a, u, s = sk.fit(sample)
        # except:
        #     print('parameterization unsuccessful, returning only histogram')
        #     return {'histogram': histo,
        #             'fit distribution': [None, None, None]}
        return {'histogram': histo,
                'fit distribution': [a, u, s]}
    return {'histogram': histo}

def monte_carlo(mc_number, He, He_s=0,
                U238=0, U235=None, Th232=0, Sm147=0,
                Ft238=1, Ft235=1, Ft232=1, Ft147=1,
                U238_s=0, U235_s=None, Th232_s=0, Sm147_s=0,
                Ft238_s=0, Ft235_s=0, Ft232_s=0, Ft147_s=0,
                U238_U235_v=0, U238_Th232_v=0, U238_Sm147_v=0,
                U235_Th232_v=0, U235_Sm147_v=0, Th232_Sm147_v=0,
                Ft238_Ft235_v=0, Ft238_Ft232_v=0, Ft238_Ft147_v=0,
                Ft235_Ft232_v=0, Ft235_Ft147_v=0, Ft232_Ft147_v=0,
                histogram=False, parameterize=False):
    '''
    Performs Monte Carlo modeling for a (U-Th)/He dataset. Note that
    while all radionuclides are optional and default to 0, at least one
    *MUST* be >0.
    
    Parameters
    ----------
    mc_number : int
        The number of monte carlo cycles to perform
        
    He : float
        The amount of helium measured. The exact units are unimportant
        as long as they are consistent with the other radionuclides
    
    He_s : float, optional
        The uncertainty in He, in the same units
        
    U238 : float, optional
        The amount of 238U measured
        
    U238_s : float, optional
        The uncertainty in 238U
        
    U235 : float, optional
        The amount of 2385U measured. If no 235U is provided, it is assumed
        to be present at the standard ratio of 137.818 238U/235U
        
    U235_s : float, optional
        The uncertainty in 235U, if measured.
        
    Th232 : float, optional
        The amount of Th232 measured
        
    Th232_s : float, optional
        The uncertainty in Th232
        
    Sm147 : float, optional
        The amount of Sm147 measured
        
    Sm147_s : float, optional
        The uncertainty in Sm147
    
    Ft238 : float, optional
        The alpha ejection correction for 238U. Should be between 0-1.
        
    Ft238_s : float, optional
        The uncertainty in Ft238
        
    Ft235 : float, optional
        The alpha ejection correction for 235U. Should be between 0-1.
        
    Ft235_s : float, optional
        The uncertainty in Ft235
        
    Ft232 : float, optional
        The alpha ejection correction for 232Th. Should be between 0-1.
        
    Ft232_s : float, optional
        The uncertainty in Ft232
        
    Ft147 : float, optional
        The alpha ejection correction for 147Sm. Should be between 0-1.
        
    Ft147_s : float, optional
        The uncertainty in Ft147
        
    U238_U235_v : float, optional
        The covariance between U238 and U235
    
    U238_Th232_v : float, optional
        The covariance between U238 and Th232
    
    U238_Sm147_v : float, optional
        The covariance between U238 and Sm147
    
    U235_Th232_v : float, optional
        The covariance between U235 and Th232
    
    U235_Sm147_v : float, optional
        The covariance between U235 and Sm147
    
    Th232_Sm147_v : float, optional
        The covariance between Th232 and Sm147
    
    Ft238_Ft235_v : float, optional
        The covariance between Ft238 and Ft235
    
    Ft238_Ft232_v : float, optional
        The covariance between Ft238 and Ft232
    
    Ft238_Ft147_v : float, optional
        The covariance between Ft238 and Ft147
    
    Ft235_Ft232_v : float, optional
        The covariance between Ft235 and Ft232
    
    Ft235_Ft147_v : float, optional
        The covariance between Ft235 and Ft147
    
    Ft232_Ft147_v : float, optional
        The covariance between Ft232 and Ft147
        
    histogram : bool, optional
        Whether to generate a histogram for the Monte Carlo results
        
    parameterize : bool, optional
        Whether to parameterize the histogram with a skew-normal distribution.
        If histogram is False, parameterize will not be used
        
    Returns
    -------
    Produces a dictionary with entries for the raw and alpha-ejection corrected
    results. Each entry is a dictionary with mean, +/- 68% CI, and the number of
    successful cycles. If histogram is true, the histogram for each is returned 
    as a list of the bin centers and a list of the number of samples within the 
    bin. The parameters are associated with the key "fit distribution", and are 
    a list of the three parameters (skewness, location, scale) to calculate a 
    skew-normal distribution
    '''
    # Helium is always univariate, so generate normal sample here
    He = np.random.normal(He, He_s, mc_number)
    
    # Generate radionuclides covariance matrix (3x3 in most cases, 4x4 for U235 measured)
    # and corresponding multivariate normal arrays
    if U235 == None:
        rad_means = [U238, Th232, Sm147]
        rad_cov = [[U238_s**2, U238_Th232_v, U238_Sm147_v],
                   [U238_Th232_v, Th232_s**2, Th232_Sm147_v],
                   [U238_Sm147_v, Th232_Sm147_v, Sm147_s**2]]
        rads = np.random.multivariate_normal(rad_means, rad_cov, mc_number).T
        U238 = rads[0]
        Th232 = rads[1]
        Sm147 = rads[2]
        U235 = U238/137.818
    else:
        rad_means = [U238, U235, Th232, Sm147]
        rad_cov = [[U238_s**2, U238_U235_v, U238_Th232_v, U238_Sm147_v],
                   [U238_U235_v, U235_s**2, U235_Th232_v, U235_Sm147_v],
                   [U238_Th232_v, U235_Th232_v, Th232_s**2, Th232_Sm147_v],
                   [U238_Sm147_v, U235_Sm147_v, Th232_Sm147_v, Sm147_s**2]]
        rads = np.random.multivariate_normal(rad_means, rad_cov, mc_number).T
        U238 = rads[0]
        U235 = rads[1]
        Th232 = rads[2]
        Sm147 = rads[3]
    
    # Generate Ft covariance matrix and multivariate normal arrays
    Ft_means = [Ft238, Ft235, Ft232, Ft147]
    Ft_covs = [[Ft238_s**2, Ft238_Ft235_v, Ft238_Ft232_v, Ft238_Ft147_v],
               [Ft238_Ft235_v, Ft235_s**2, Ft235_Ft232_v, Ft235_Ft147_v],
               [Ft238_Ft232_v, Ft235_Ft232_v, Ft232_s**2, Ft232_Ft147_v],
               [Ft238_Ft147_v, Ft235_Ft147_v,Ft232_Ft147_v, Ft147_s**2]]
    Fts = np.random.multivariate_normal(Ft_means, Ft_covs, mc_number).T
    Ft238 = Fts[0]
    Ft235 = Fts[1]
    Ft232 = Fts[2]
    Ft147 = Fts[3]
    
    # Call the get_date function to calculate a date for each
    # of the randomly generated data
    MonteCarlo_t = get_date(He, U238, U235, Th232, Sm147,
                            Ft238, Ft235, Ft232, Ft147)
        
    # Create a dictionary to save results to
    mc_results = {'raw date': {},
                  'corrected date': {}}
    # Produce statistics and save
    for t in ['raw date', 'corrected date']:
        mean = np.mean(MonteCarlo_t[t])
        CIs_68 = np.percentile(MonteCarlo_t[t], [15.865, 84.135])
        CIs_95 = np.percentile(MonteCarlo_t[t], [2.5, 97.5])
        mc_results[t]['mean'] = mean
        mc_results[t]['+68% CI'] = CIs_68[1]
        mc_results[t]['-68% CI'] = CIs_68[0]
        mc_results[t]['+95% CI'] = CIs_95[1]
        mc_results[t]['-95% CI'] = CIs_95[0]
        # Report how many of the Monte Carlo simulations converged to
        # an age. For highly uncertain input data, this may be significantly
        # lower than the number of requested cycles.
        mc_results[t]['cycles'] = len(MonteCarlo_t[t])
        if histogram:
            histogram_out = make_histogram(MonteCarlo_t[t], parameterize)
            mc_results[t]['histogram'] = histogram_out['histogram']
            if parameterize:
                mc_results[t]['a'] = histogram_out['fit distribution'][0]
                mc_results[t]['u'] = histogram_out['fit distribution'][1]
                mc_results[t]['s'] = histogram_out['fit distribution'][2]
    
    return mc_results